//
//  ContainerViewController.m
//  ListMe
//
//  Created by Nick Aiwazian on 10/30/13.
//  Copyright (c) 2013 com.nickaiwazian. All rights reserved.
//

#import "ContainerViewController.h"
#import "ListViewController.h"
#import "MenuViewController.h"

@interface ContainerViewController () <HandleMenuButtonDelegate, UIGestureRecognizerDelegate, UIDynamicAnimatorDelegate>

// Gesture Recognizers
@property (nonatomic, strong) UIScreenEdgePanGestureRecognizer *leftScreenEdgeGestureRecognizer;
@property (nonatomic, strong) UIScreenEdgePanGestureRecognizer *rightScreenEdgeGestureRecognizer;

// Animation
@property (nonatomic, strong) UIDynamicAnimator *animator;
@property (nonatomic, strong) UIGravityBehavior *gravityBehaviour;
@property (nonatomic, strong) UIPushBehavior* pushBehavior;
@property (nonatomic, strong) UIAttachmentBehavior *panAttachmentBehaviour;

// Controllers
@property (nonatomic, strong) ListViewController *listViewController;
@property (nonatomic, strong) MenuViewController *menuViewController;

@property (nonatomic, weak) IBOutlet UIView *contentView;

@property (nonatomic, assign, getter = isMenuOpen) BOOL menuOpen;

@end

@implementation ContainerViewController

/**
 * Description: Set up the gesture recognizers
 */

- (void)viewDidLoad {
    
    [super viewDidLoad];
    
    // Set up left screen edge pan gesture recognizer
    self.leftScreenEdgeGestureRecognizer = [[UIScreenEdgePanGestureRecognizer alloc] initWithTarget:self action:@selector(handleScreenEdgePan:)];
    self.leftScreenEdgeGestureRecognizer.edges = UIRectEdgeLeft;
    self.leftScreenEdgeGestureRecognizer.delegate = self;
    [self.view addGestureRecognizer:self.leftScreenEdgeGestureRecognizer];

    // Set up right screen edge pan gesture recognizer
    self.rightScreenEdgeGestureRecognizer = [[UIScreenEdgePanGestureRecognizer alloc] initWithTarget:self action:@selector(handleScreenEdgePan:)];
    self.rightScreenEdgeGestureRecognizer.edges = UIRectEdgeRight;
    self.rightScreenEdgeGestureRecognizer.delegate = self;
    [self.view addGestureRecognizer:self.rightScreenEdgeGestureRecognizer];
}

/**
 * Description: Set up the shadow on the list view
 */

-(void)viewDidAppear:(BOOL)animated {
    
    [super viewDidAppear:animated];
    
    // Important to call this only after our view is on screen (i.e. we can trust the view geometry).
    [self setupContentViewControllerAnimatorProperties];
    
    self.contentView.layer.shadowColor = [[UIColor blackColor] CGColor];
    self.contentView.layer.shadowOpacity = 1.0f;
    self.contentView.layer.shadowPath = [[UIBezierPath bezierPathWithRect:self.contentView.bounds] CGPath];
    self.contentView.layer.shadowOffset = CGSizeZero;
    self.contentView.layer.shadowRadius = 5.0f;
    
}

- (void)setupContentViewControllerAnimatorProperties {
    
    NSAssert(self.animator == nil, @"Animator is not nil – setupContentViewControllerAnimatorProperties likely called twice.");
    
    self.animator = [[UIDynamicAnimator alloc] initWithReferenceView:self.view];
    
    UICollisionBehavior *collisionBehaviour = [[UICollisionBehavior alloc] initWithItems:@[self.contentView]];
    
    // Need to create a boundary that lies to the left off of the right edge of the screen.
    [collisionBehaviour setTranslatesReferenceBoundsIntoBoundaryWithInsets:UIEdgeInsetsMake(0, 0, 0, -280)];
    [self.animator addBehavior:collisionBehaviour];
    
    self.gravityBehaviour = [[UIGravityBehavior alloc] initWithItems:@[self.contentView]];
    self.gravityBehaviour.gravityDirection = CGVectorMake(-1, 0);
    [self.animator addBehavior:self.gravityBehaviour];
    
    UIDynamicItemBehavior *itemBehavior = [[UIDynamicItemBehavior alloc] initWithItems:@[self.contentView]];
    itemBehavior.elasticity = 0.0f;

    [self.animator addBehavior:itemBehavior];
}

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    
    [super prepareForSegue:segue sender:sender];
    
    if ([segue.identifier isEqualToString:@"listViewController"]) {
        self.listViewController = (ListViewController *)[segue.destinationViewController topViewController];
        self.listViewController.menuButtonDelegate = self;
    }
    else if([segue.identifier isEqualToString:@"menuViewController"]) {
        self.menuViewController = (MenuViewController *)[segue.destinationViewController topViewController];
    }
}

#pragma mark - UIGestureRecognizerDelegate Methods

- (BOOL)gestureRecognizer:(UIGestureRecognizer *)gestureRecognizer shouldReceiveTouch:(UITouch *)touch {
    if (gestureRecognizer == self.leftScreenEdgeGestureRecognizer && self.isMenuOpen == NO) {
        return YES;
    }
    else if (gestureRecognizer == self.rightScreenEdgeGestureRecognizer && self.isMenuOpen == YES) {
        return YES;
    }
    
    return NO;
}

- (void)handleScreenEdgePan:(UIScreenEdgePanGestureRecognizer *)gestureRecognizer {
    NSLog(@"Touched me!!!");

    CGPoint location = [gestureRecognizer locationInView:self.view];
    location.y = CGRectGetMidY(self.contentView.bounds);
    
    if (gestureRecognizer.state == UIGestureRecognizerStateBegan) {
        [self.animator removeBehavior:self.gravityBehaviour];
        
        self.panAttachmentBehaviour = [[UIAttachmentBehavior alloc] initWithItem:self.contentView attachedToAnchor:location];
        [self.animator addBehavior:self.panAttachmentBehaviour];
    }
    else if (gestureRecognizer.state == UIGestureRecognizerStateChanged) {
        self.panAttachmentBehaviour.anchorPoint = location;
    }
    else if (gestureRecognizer.state == UIGestureRecognizerStateEnded) {
        [self.animator removeBehavior:self.panAttachmentBehaviour], self.panAttachmentBehaviour = nil;
        
        CGPoint velocity = [gestureRecognizer velocityInView:self.view];
        
        if (velocity.x > 0) {
            // Open menu
            self.menuOpen = YES;
            
            self.gravityBehaviour.gravityDirection = CGVectorMake(1, 0);
        }
        else {
            // Close menu
            self.menuOpen = NO;
            
            self.gravityBehaviour.gravityDirection = CGVectorMake(-1, 0);
        }
        
        [self.animator addBehavior:self.gravityBehaviour];
        
        self.pushBehavior.pushDirection = CGVectorMake(velocity.x / 10.0f, 0);
        self.pushBehavior.active = YES;
    }
}

#pragma mark - Delegate

- (void)listViewControllerDidPressMenuButton:(ListViewController *)viewController {
    NSLog(@"Pressed button");
    [self toggleMenuView];
    [self.menuViewController.tableView reloadData];
}

/**
 * Description: Toggle whether the menu is open by setting the gravity in either direction
 */

- (void)toggleMenuView {

    if(self.isMenuOpen == YES) {

        self.menuOpen = NO;
        self.gravityBehaviour.gravityDirection = CGVectorMake(-1, 0);
    }
    else {
        
        self.menuOpen = YES;
        self.gravityBehaviour.gravityDirection = CGVectorMake(1, 0);
    }
}

@end
