//
//  CoreDataHelper.m - Singleton class that wraps Core Data
//  ListMe
//
//  Created by Nick Aiwazian on 10/31/13.
//  Copyright (c) 2013 com.nickaiwazian. All rights reserved.
//

#import "CoreDataHelper.h"

@interface CoreDataHelper()

@property (strong, nonatomic) NSManagedObjectModel *managedObjectModel;
@property (strong, nonatomic) NSPersistentStoreCoordinator *persistentStoreCoordinator;
@property (strong, nonatomic) NSURL *storeURL;
@property (strong, nonatomic) NSURL *applicationDocumentsDirectory;

@end

/**
 * TODO: Fix the case where we don't load default data
 * TODO: Factor out all configuration into initialize so rest of class is model agnostic
 */

@implementation CoreDataHelper

#pragma mark - Singleton

// Singleton instance
static CoreDataHelper *sharedCoreDataHelper;
static BOOL initialized = NO;

/**
 * Description: Return the singleton instance
 */

+ (CoreDataHelper *)sharedCoreDataHelper {
    return sharedCoreDataHelper;
}

#pragma mark - Initialization

/**
 * Description: Sets up the singleton instance.  If in DEBUG mode, load test data.
 *
 *      The runtime sends initialize to each class in a program exactly one time just before the
 *      class, or any class that inherits from it, is sent its first message from within the program.
 *      (Thus the method may never be invoked if the class is not used.) The runtime sends the
 *      initialize message to classes in a thread-safe manner. Superclasses receive this message
 *      before their subclasses.
 */

+ (void)initialize {

    if( !initialized ) {
        
        #ifdef DEBUG
            sharedCoreDataHelper = [[CoreDataHelper alloc] initWith:YES];
        #else
            sharedCoreDataHelper = [[CoreDataHelper alloc] initWith:NO];
        #endif
        
        initialized = YES;
    }
}

/**
 * Description: Internal initialization if singleton has not already been initialized
 */

- (CoreDataHelper *)initWith:(BOOL)loadData {

    if( !initialized ) {

        self = [super init];

        if ( self ) {
            
            if( loadData ) {
                
                if( ![[NSFileManager defaultManager] fileExistsAtPath:[self.storeURL path]] ) {

                    NSError *error;
                    NSURL *preloadURL = [NSURL fileURLWithPath:[[NSBundle mainBundle]
                                                                pathForResource:ModelURLName
                                                                ofType:ModelType]];
                    
                    if( ![[NSFileManager defaultManager] copyItemAtURL:preloadURL
                                                                 toURL:self.storeURL
                                                                 error:&error] ) {
                        
                        [NSException raise:@"Unable to load preloaded data"
                                    format:ErrorFormat, error.description];
                    }
                }
                
                [self.managedObjectContext setPersistentStoreCoordinator:self.persistentStoreCoordinator];
            }
        }
    }
    
    return self;
}

#pragma mark - Properties

/**
 * Description: Create the managedObjectContext
 */

- (NSManagedObjectContext *)managedObjectContext {

    if( !_managedObjectContext ) {

        _managedObjectContext = [[NSManagedObjectContext alloc] init];
    }
    
    return _managedObjectContext;
}

/**
 * Description: Get the local documents directory
 */

- (NSURL *)applicationDocumentsDirectory {
    if( !_applicationDocumentsDirectory ) {
        
        _applicationDocumentsDirectory = [[[NSFileManager defaultManager] URLsForDirectory:NSDocumentDirectory inDomains:NSUserDomainMask] lastObject];
    }
    
    return _applicationDocumentsDirectory;
}

/**
 * Description: Get the MOM from file location
 */

- (NSManagedObjectModel *)managedObjectModel {

    if( !_managedObjectModel) {

        NSURL *modelURL = [[NSBundle mainBundle] URLForResource:ModelURLName
                                                  withExtension:ModelURLNameExtension];
        
        _managedObjectModel = [[NSManagedObjectModel alloc] initWithContentsOfURL:modelURL];
    }
    
    return _managedObjectModel;
}

/**
 * Description: Create the storeURL
 */

- (NSURL *)storeURL {

    if( !_storeURL ) {

        _storeURL = [self.applicationDocumentsDirectory URLByAppendingPathComponent:PersistentStoreFileName];
    }
    
    return _storeURL;
}

/**
 * Description: Create the persistent store
 */

- (NSPersistentStoreCoordinator *)persistentStoreCoordinator {
    
    if( !_persistentStoreCoordinator ) {

        _persistentStoreCoordinator = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel:self.managedObjectModel];

        NSError *error;

        if ( ![self.persistentStoreCoordinator addPersistentStoreWithType:NSSQLiteStoreType
                                                            configuration:nil
                                                                      URL:self.storeURL
                                                                  options:nil
                                                                    error:&error] ) {

            [NSException raise:@"Unable to create persistent store"
                        format:ErrorFormat, error.description];
        }
    }
    
    return _persistentStoreCoordinator;
}

#pragma mark - Queries

/**
 * Description: Given an entity and a predicate, figures out how many records WOULD have been returned
 * Throws an exception if there is an error with the request
 */

- (NSUInteger)executeCountOfCoreDataQueryFor:(NSString *)entity with:(NSPredicate *)predicate {

    NSUInteger count = 0;
    NSError *error;
    
    NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
    
    NSEntityDescription *entityDescription = [NSEntityDescription entityForName:entity
                                                         inManagedObjectContext:self.managedObjectContext];
    
    [fetchRequest setEntity:entityDescription];
    [fetchRequest setPredicate:predicate];
    
    count = [self.managedObjectContext countForFetchRequest:fetchRequest error:&error];
    
    if( error ) {
        [NSException raise:@"Error code is nonzero for core data count request"
                    format:ErrorFormat, error.description];
    }

    return count;
}

/**
 * Description: Given an entity, predicate, and sort order, gets the results of the request
 * Throws an exception if there is an error with the request
 */

- (NSArray *)executeCoreDataQueryFor:(NSString *)entity with:(NSPredicate *)predicate andSort:(NSArray *)sortOrders {
    
    NSError *error;

    NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
    NSEntityDescription *entityDescription = [NSEntityDescription entityForName:entity inManagedObjectContext:self.managedObjectContext];
    
    [fetchRequest setSortDescriptors:sortOrders];
    [fetchRequest setEntity:entityDescription];
    [fetchRequest setPredicate:predicate];
    
    NSArray *results = [self.managedObjectContext executeFetchRequest:fetchRequest error:&error];
    
    if( error ) {
        [NSException raise:@"Error code is nonzero for core data fetch request"
                    format:ErrorFormat, error.description];
    }

    return results;
}

/**
 * Description: Execute a query on 'entity' core data with a specific column specified as 'sortKey'
 */

- (NSArray *)executeCoreDataQueryWithAlphabeticalSortFor:(NSString *)entity with:(NSPredicate *)predicate andSortKey:(NSString *)sortKey {

    NSSortDescriptor *sortByNameAscending = [[NSSortDescriptor alloc] initWithKey:sortKey ascending:YES];
    NSArray *sortDescriptors = [[NSArray alloc] initWithObjects:sortByNameAscending, nil];
    
    return [self executeCoreDataQueryFor:entity with:predicate andSort:sortDescriptors];
}

#pragma mark - Add / Remove individual objects

/**
 * Description: Add a new object and save context
 */

- (void)addManagedObject:(NSManagedObject *)object
{
    [self.managedObjectContext insertObject:object];
    
    [self saveContext];
}

/**
 * Description: Delete an object and save context
 */

- (void)deleteManagedObject:(NSManagedObject *)object
{
    [self.managedObjectContext deleteObject:object];
    
    [self saveContext];
}

#pragma mark - Save / Purge

/**
 * Description: Save all pending changes to core data
 */

- (void)saveContext
{
    NSError *error;
    
    if ( self.managedObjectContext ) {
        
        if ( [self.managedObjectContext hasChanges] && ![self.managedObjectContext save:&error] ) {

            [NSException raise:@"Error when trying to save core data" format:ErrorFormat, [error userInfo]];
        }
    }
}

/**
 * Description: Clears out all core data records.  Use VERY carefully!
 */

- (void)purgeCoreData {
    
    NSArray *stores = [self.persistentStoreCoordinator persistentStores];
    
    for( NSPersistentStore *store in stores ) {

        NSError *storeError;

        [self.persistentStoreCoordinator removePersistentStore:store error:&storeError];

        if( storeError ) {
            [NSException raise:@"Error when trying to purge core data stores"
                        format:ErrorFormat, storeError.description];
        }

        NSError *fileError;
        
        [[NSFileManager defaultManager] removeItemAtPath:store.URL.path error:&fileError];

        if( fileError ) {
            [NSException raise:@"Error when trying to purge core data stores"
                        format:ErrorFormat, fileError.description];
        }
    }
}

@end
