//
//  ListMeMasterViewController.m
//  ListMe
//
//  Created by Nick Aiwazian on 10/1/13.
//  Copyright (c) 2013 com.nickaiwazian. All rights reserved.
//

#import "ListViewController.h"
#import "List.h"
#import "ListItem.h"
#import "DetailViewController.h"
#import "DateRanges.h"
#import "DateRange.h"
#import "CoreDataHelper.h"
#import "ListItemTableCell.h"
#import "UserDefaultsWrapper.h"

@interface ListViewController () <ListItemTableCellDelegate>

@property (strong, nonatomic) NSArray *standardDateRanges; // DateRange
@property (strong, nonatomic) CoreDataHelper *sharedCoreDataHelper;
@property (strong, nonatomic) List *list;

@end

@implementation ListViewController

@synthesize list = _list;

#pragma mark - Properties

/**
 * Description: Get the standard date ranges array
 */

- (NSArray *)standardDateRanges {

    if( !_standardDateRanges ) {

        DateRanges *dateRanges = [[DateRanges alloc] init];
        _standardDateRanges = dateRanges.standardDateRanges;
    }
    
    return _standardDateRanges;
}

/**
 * Description: Get the SharedCoreDataHelper singleton
 */

- (CoreDataHelper *)sharedCoreDataHelper {
    
    if( !_sharedCoreDataHelper ) {
        _sharedCoreDataHelper = CoreDataHelper.sharedCoreDataHelper;
    }
    
    return _sharedCoreDataHelper;
}

/**
 * Description: Get the list to view.  If no list is set, first check user defaults and then just return
 * first list
 */

- (List *)list {
    
    if( !_list ) {
        
//        NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
//        NSURL *listUri = (NSURL *)[userDefaults objectForKey:DefaultListKey];
        //BUGBUG: Fix this later
//        if ( listUri ) {
            
//            NSPersistentStoreCoordinator *store = self.sharedCoreDataHelper.managedObjectContext.persistentStoreCoordinator;
//            NSManagedObjectID *objectId = [store managedObjectIDForURIRepresentation:listUri];
//            _list = (List *)[self.sharedCoreDataHelper.managedObjectContext objectWithID:objectId];
//        }
//        else {
            
            NSArray *results = [self.sharedCoreDataHelper executeCoreDataQueryWithAlphabeticalSortFor:ListEntity with:nil andSortKey:ListName];
            _list = [results firstObject];
//        }
    }
    
    return _list;
}

/**
 * Description: Set the list and update NSUserDefaults
 */

- (void)setList:(List *)list {
    
    _list = list;
    
    [self updateUI];
    
    // Set the DefaultListKey in user defaults
    NSURL *listUri = [[list objectID] URIRepresentation];
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setURL:listUri forKey:DefaultListKey];
    [userDefaults synchronize];
}

#pragma mark - UI

/**
 * Description: Update the title and reload the table
 */

- (void)updateUI {
    
    self.title = self.list.name;
    
    [self.tableView reloadData];
}

/**
 * Description: Handle menu press action
 */

- (IBAction)menuButtonPressed:(UIBarButtonItem *)sender {
    
    NSLog(@"Pressed");
    [self.menuButtonDelegate listViewControllerDidPressMenuButton:self];
}

#pragma mark - Table View Controller

/**
 * Description: Gets number of sections by finding out how many dateRanges 
 * have nonzero count.
 * TODO: Add support for empty date range
 */

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    int numberOfSections = 0;

    // Iterate through all date ranges and sum all ranges that have nonzero rows
    for( DateRange *dr in self.standardDateRanges ) {
        
        NSUInteger count = [self getCountOfListItemEntriesFrom:dr.startDate to:dr.endDate];
        if( count > 0 )
            numberOfSections++;
    }
  
    return numberOfSections;
}

/**
 * Description: Gets number of rows from dateRanges array
 */

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    
    NSUInteger index = [self findIndexInDateGrouping:section];
    DateRange *dr = [self.standardDateRanges objectAtIndex:index];
    NSUInteger count = [self getCountOfListItemEntriesFrom:dr.startDate to:dr.endDate];
    
    return count;
}

/**
 * Description: Uses helper functions to display rows based on indexPath
 */
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    static NSString *CellIdentifier = @"listItemTableCell";
    ListItemTableCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];

    if ( !cell ) {
        NSArray *nib = [[NSBundle mainBundle] loadNibNamed:@"ListItemTableCell" owner:self options:nil];
        cell = [nib objectAtIndex:0];
    }

    ListItem *listItem = [self getListItemAtIndexPath:indexPath];

    NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
    [formatter setDateFormat:DateFormat];
    [formatter setTimeZone:[NSTimeZone timeZoneWithName:TimeZone]];

    NSString *dueDate = [formatter stringFromDate:listItem.dueDate];
    
    cell.name.text = listItem.name;
    cell.dueDate.text = [NSString stringWithFormat:@"%@%@", DueDatePrefix, dueDate];
    cell.completed = listItem.completed && [listItem.completed boolValue] == TRUE ? YES : NO;
    cell.completionDelegate = self;
    cell.listItem = listItem;
        
    // Set up the appropriate color, red is for overdue
    UIColor *listItemColor = [listItem.dueDate compare:[NSDate dateWithNoTime]] == NSOrderedAscending ?
        [UIColor redColor] : [UIColor darkTextColor];
    
    cell.name.textColor = listItemColor;
    cell.dueDate.textColor = listItemColor;

    return cell;
}

/**
 * Description: Looks up the header from DateGrouping Array
 */
- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {

    NSUInteger index = [self findIndexInDateGrouping:section];
    DateRange *dr = [self.standardDateRanges objectAtIndex:index];
    
    return dr.name;
}

#pragma mark - Navigation

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    NSLog(@"Selected this row");
    [self performSegueWithIdentifier:@"showDetailView" sender:self];
}

// In a story board-based application, you will often want to do a little preparation before navigation

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {

    if ([[segue identifier] isEqualToString:@"showDetailView"]) {
        NSIndexPath *indexPath = [self.tableView indexPathForSelectedRow];
        ListItem *item = [self getListItemAtIndexPath:indexPath];
        [[segue destinationViewController] setParentListItem:item];
    }
}

#pragma mark Helper Methods

/**
 * Description: Gets the number of list items in core data
 * between startDate and endDate
 */
- (NSUInteger) getCountOfListItemEntriesFrom:(NSDate *)startDate to:(NSDate *)endDate {
    
    NSPredicate *betweenPredicate = [NSPredicate predicateWithFormat: @"(dueDate >= %@) AND (dueDate <= %@)", startDate, endDate];
    NSPredicate *listPredicate = [NSPredicate predicateWithFormat:@"list.name = %@", self.list.name];
    
    NSCompoundPredicate *fullPredicate = [[NSCompoundPredicate alloc] initWithType:NSAndPredicateType subpredicates:[[NSArray alloc] initWithObjects:betweenPredicate, listPredicate, nil]];

    NSUInteger count = [self.sharedCoreDataHelper executeCountOfCoreDataQueryFor:ListItemEntity with:fullPredicate];
    
    return count;
}

/**
 * Description: Since not all sections in UI contain tasks, 
 * need to map UI sections to actual sections in the DateGrouping
 * Array
 */

- (NSUInteger) findIndexInDateGrouping:(long)indexInTable {
    
    __block int sectionCount = 0;
    __block NSUInteger foundIndex = -1;
    
    [self.standardDateRanges enumerateObjectsUsingBlock:^(DateRange *dateRange, NSUInteger index, BOOL *stop) {

        NSUInteger count = [self getCountOfListItemEntriesFrom:dateRange.startDate to:dateRange.endDate];
        
        if( count > 0 )
            sectionCount++;
        
        if( sectionCount == indexInTable + 1 ) {
            foundIndex = index;
            *stop = YES;
        }
    }];
    
    return foundIndex;
}

/**
 * Description: Given an indexPath, return the list item at the index path
 */
- (ListItem *) getListItemAtIndexPath:(NSIndexPath *)indexPath {
    
    NSUInteger index = [self findIndexInDateGrouping:indexPath.section];
    DateRange *dr = [self.standardDateRanges objectAtIndex:index];
    
    NSPredicate *betweenPredicate = [NSPredicate predicateWithFormat: @"(dueDate >= %@) AND (dueDate <= %@)", dr.startDate, dr.endDate];
    NSPredicate *listPredicate = [NSPredicate predicateWithFormat:@"list.name = %@", self.list.name];
    
    NSCompoundPredicate *fullPredicate = [[NSCompoundPredicate alloc] initWithType:NSAndPredicateType subpredicates:[[NSArray alloc] initWithObjects:betweenPredicate, listPredicate, nil]];
    
    NSSortDescriptor *sortByDueDateAscending = [[NSSortDescriptor alloc] initWithKey:@"dueDate" ascending:YES];
    NSSortDescriptor *sortByNameAscending = [[NSSortDescriptor alloc] initWithKey:@"name" ascending:YES];
    NSArray *sortDescriptors = [[NSArray alloc] initWithObjects:sortByDueDateAscending, sortByNameAscending, nil];
    
    NSArray *results = [self.sharedCoreDataHelper executeCoreDataQueryFor:ListItemEntity with:fullPredicate andSort:sortDescriptors];
    
    ListItem *item = results[indexPath.row];
    return item;
}

- (void) listItemCompletionStatusUpdate:(BOOL)completionStatus forItem:(ListItem *)listItem {
    
    listItem.completed = [[NSNumber alloc] initWithBool:completionStatus];
    [self.sharedCoreDataHelper saveContext];
}

@end
