//
//  MenuViewController.m
//  ListMe
//
//  Created by Nick Aiwazian on 10/30/13.
//  Copyright (c) 2013 com.nickaiwazian. All rights reserved.
//

#import "MenuViewController.h"
#import "CoreDataHelper.h"
#import "List.h"
#import "ListViewController.h"

@interface MenuViewController ()

@property (strong, nonatomic) CoreDataHelper *sharedCoreDataHelper;
@property (strong, nonatomic) ListViewController *listViewController;
@property (strong, nonatomic) List *currentList;

@end

@implementation MenuViewController

@synthesize currentList = _currentList;

#pragma mark - UI

/**
 * Description: Make sure all delete buttons are hidden and reload table data
 */

- (void)updateUI {
    
    [self hideAllDeleteButtons:self.view];
    [self.tableView reloadData];
}

/**
 * Description: Update the list view to passed in list
 */

- (void)updateListViewUI:(List *)list {
    
    [self.listViewController setList:list];
}

/**
 * Description: Ask the listView to show itself again
 */

- (void)switchToListView {
    
    [self.listViewController.menuButtonDelegate listViewControllerDidPressMenuButton:self.listViewController];
}

/**
 * Description: Recursively find all delete buttons and make sure they are hidden
 */

- (void)hideAllDeleteButtons:(UIView *) view {

    for(UIView *v in [view subviews]) {
        
        if( [v isKindOfClass:[UIButton class]] ) {
        
            UIButton *button = (UIButton *)v;
            button.hidden = YES;
        }
        else {

            [self hideAllDeleteButtons:v];
        }
    }
}

#pragma mark - Targets

/**
 * Description: After typing in a new list, add the list to core data and set it as
 * the current list
 */

- (IBAction)addListTextFieldExited:(UITextField *)sender {
    
    UITextField *textField = sender;
    
    List *newList = [NSEntityDescription insertNewObjectForEntityForName:ListEntity inManagedObjectContext:self.sharedCoreDataHelper.managedObjectContext];
    newList.name = textField.text;
    newList.datedList = [[NSNumber alloc]initWithBool:YES];
    
    [self.sharedCoreDataHelper addManagedObject:newList];
    textField.text = textField.placeholder;
    [self updateUI];
    self.currentList = newList;
}

/**
 * Description: On delete, pop up a confirmation alert
 */

- (void)deleteButtonClicked:(UIButton *)sender {

    UITableViewCell *cell = (UITableViewCell *)sender.superview.superview.superview;
    NSString *listName = cell.textLabel.text;
    
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:AlertConfirmationTitle message:[NSString stringWithFormat:AlertText, listName] delegate:self cancelButtonTitle:AlertNoButtonText otherButtonTitles:AlertYesButtonText, nil];
    alert.tag = sender.tag;
    [alert show];
}

/**
 * Description: On long press, hide all delete buttons and unhide the current delete button
 */

- (void)longPress:(UILongPressGestureRecognizer *)recognizer {
    UIView *parentView = recognizer.view;
    
    [self hideAllDeleteButtons:self.view];
    
    for( UIView *view in [parentView subviews] ) {
        
        if( [view isKindOfClass:[UIButton class]] ) {

            view.hidden = NO;
        }
    }
}

/**
 * Description: Delete confirmation target.  Use the tag as an index into the list
 * results to know which one user wants to delete.
 */

- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
    // User confirmed they want to delete
    if( buttonIndex == 1 ) {
        
        NSArray *results = [self.sharedCoreDataHelper executeCoreDataQueryWithAlphabeticalSortFor:ListEntity with:nil andSortKey:ListName];
        
        List *list = results[alertView.tag];
        
        [CoreDataHelper.sharedCoreDataHelper deleteManagedObject:list];
        [self setCurrentList:nil];
    }
    [self updateUI];
    
}

#pragma mark - Properties

/**
 * Description: Get coreDataHelper from singleton
 */

- (CoreDataHelper *)sharedCoreDataHelper {

    if( !_sharedCoreDataHelper ) {
        
        _sharedCoreDataHelper = [CoreDataHelper sharedCoreDataHelper];
    }
    
    return _sharedCoreDataHelper;
}

/**
 * Description: If currentList is not set, give me the first list FIXFIX
 */

- (List *)currentList {
    
    if( !_currentList ) {
        
        NSArray *results = [self.sharedCoreDataHelper executeCoreDataQueryWithAlphabeticalSortFor:ListEntity with:nil andSortKey:ListName];
        _currentList = [results firstObject];
    }
    
    return _currentList;
}

/**
 * Description: Set currentList and update ListView UI each time
 */

- (void)setCurrentList:(List *)currentList {

    _currentList = currentList;
    [self updateListViewUI:self.currentList];
}

/**
 * Description: Walk the view controller hierarchy to find the ListViewController
 */

- (ListViewController *)listViewController {
    
    if( !_listViewController ) {

        NSArray *higherViewControllers = [self.parentViewController.parentViewController childViewControllers];
        
        for( UIViewController *uivc in higherViewControllers ) {
    
            NSArray *siblingViewControllers = [uivc childViewControllers];
            
            for( UIViewController *uivc in siblingViewControllers ) {
                
                if( [uivc isKindOfClass:[ListViewController class]] ) {
                    _listViewController = (ListViewController *)uivc;
                    break;
                }
            }
        }
    }
    
    return _listViewController;
}

#pragma mark - Table View Controller

/**
 * Description: Get the actual list names FIXFIX
 */

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *cellIdentifier = @"list";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellIdentifier];
    
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault
                                      reuseIdentifier:cellIdentifier];
    }
    
    NSArray *results = [self.sharedCoreDataHelper executeCoreDataQueryWithAlphabeticalSortFor:ListEntity
                                                                                         with:nil
                                                                                   andSortKey:ListName];
    
    // Set the text
    List *list = results[indexPath.row];
    cell.textLabel.text = list.name;
    
    // Create the delete button
    UIButton *deleteButton = [UIButton buttonWithType:UIButtonTypeRoundedRect];
    [deleteButton addTarget:self action:@selector(deleteButtonClicked:) forControlEvents:UIControlEventTouchUpInside];
    
    CGRect bounds = cell.contentView.bounds;
    
    deleteButton.frame = CGRectMake(bounds.size.width - 200,0,150,44);
    [deleteButton setTitle:@"Delete" forState:UIControlStateNormal];
    deleteButton.hidden = YES;
    deleteButton.tag = indexPath.row;
    [cell.contentView addSubview:deleteButton];
    
    UILongPressGestureRecognizer *longpr =
    [[UILongPressGestureRecognizer alloc] initWithTarget:self action:@selector(longPress:)];
    [cell.contentView addGestureRecognizer:longpr];
    
    return cell;
}

/**
 * Description: Once a user selects a list, tell the ListViewController to update
 */

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    
    NSArray *results = [self.sharedCoreDataHelper executeCoreDataQueryWithAlphabeticalSortFor:ListEntity
                                                                                         with:nil
                                                                                   andSortKey:ListName];
    self.currentList = results[indexPath.row];
    [self switchToListView];
}

/**
 * Description: Get the number of lists by executing a count request against core data
 */

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {

    NSUInteger count = [self.sharedCoreDataHelper executeCountOfCoreDataQueryFor:ListEntity with:nil];

    return count;
}

/**
 * Description: Hard coded to one since this is the list of lists
 */

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    
    return 1;
}

@end
